open Printf

(* ------------------------------------------------------------------------- *)

(* Parse the command line. *)

let print_type = ref false;;
let print_term = ref false;;


let filename =
  ref None

let usage =
  sprintf "Usage: %s <filename>\n" Sys.argv.(0)

let () =
  Arg.parse [
    "-pty", Arg.Set print_type, " Print type";
    "-pterm", Arg.Set print_term, " Print elaborated term";
  ] (fun name -> filename := Some name) usage

let filename =
  match !filename with
  | Some filename ->
      filename
  | None ->
      fprintf stderr "%s%!" usage;
      exit 1

(* ------------------------------------------------------------------------- *)

(* Read the file; lex; parse; internalize. *)

let prog : Terms.loc_fterm =
  let lexbuf = LexerUtil.open_in filename in
  let prog =
    try
      Parser.program Lexer.main lexbuf
    with Parser.Error ->
      Error.errorb lexbuf "Syntax error."
  in
  Internalize.program prog

(* ------------------------------------------------------------------------- *)

(* Take action. *)

let () =
  let p = Export.empty in
  (* Always typecheck *)
  let ty, prog' = Typecheck.program prog in
  (* Print type if Typecheck or Verbose *)
  let () = if !print_type then
    printf "This term is well-typed.\n%s\n%!" (Print.print_type p ty) in
  (* Print translation if Translate or Verbose *)
  let () = if !print_term then printf "%s\n%!" (Print.print_term p prog') in
  let ty', _ = Typecheck.program  prog' in
  if not (Types.equal ty ty') then
    let () = 
      eprintf "Elaboration broke the type!\n\
        Source term:\n%s\nof type\n%s:\nElaborated term:\n%s\nof type\n%s:\n%!" 
              (Print.print_term p prog) (Print.print_type p ty)
              (Print.print_term p prog') (Print.print_type p ty') in
    exit 1

